/*
  Copyright (C) 2007 Roberto Carraretto & Thiago Fassina Olimpio Carvalho

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include "ball.h"

// Read Properties
int getGameStatus (ball* b) {
	return (b->game != 0);
}

int getNumber (ball* b) {
	return (b->number);
}

vector getPosition (ball* b) {
	return (b->center);
}

float getXPosition (ball* b) {
	return (getX(&b->center));
}

float getYPosition (ball* b) {
	return (getY(&b->center));
}

vector getSpeed (ball* b) {
	return (b->speed);
}

float getXSpeed (ball* b) {
	return (getX(&b->speed));
}

float getYSpeed (ball* b) {
	return (getY(&b->speed));
}

float getRadius (ball* b) {
	return (b->radius);
}

BITMAP* getPicture (ball* b) {
	return (b->animation[b->currentFrame]);
}

float calculateSpeedAngle (ball* b) {
	vector v;
	setVector(&v, 0.0, 0.0);

	return (calculateAngle(&v, &b->speed));
}

float storeSpeedAngle (ball* b) {
	vector v;
	setVector(&v, 0.0, 0.0);

	b->speedAngle = calculateAngle(&v, &b->speed);

	return (b->speedAngle);
}

float getLastAngle (ball* b) {
	return (b->speedAngle);
}

// Write Properties
void setNumber (ball* b, int num) {
	b->number = num;
}

void setPosition (ball* b, vector* v) {
	setVector (&b->center, getX(v), getY(v));
}

void setXPosition (ball* b, float x) {
	setVector (&b->center, x, getYPosition(b));
}

void setYPosition (ball* b, float y) {
	setVector (&b->center, getXPosition(b), y);
}

void setSpeed (ball* b, vector* v) {
	setVector (&b->speed, getX(v), getY(v));
}

void setRadius (ball* b, float r) {
	b->radius = r;
}

void setPicture (ball* b, BITMAP* p, int frame) {
	b->animation[frame] = p;
}

// Other
void addPosition (ball* b, vector* v) {
	addVector(&b->center, v);
}

void addXPosition (ball* b, float value) {
	vector v;
	setVector(&v, value, 0);
	addVector(&b->center, &v);
}

void addYPosition (ball* b, float value) {
	vector v;
	setVector(&v, 0, value);
	addVector(&b->center, &v);
}

void subPosition (ball* b, vector* v) {
	subVector(&b->center, v);
}

void subXPosition (ball* b, float value) {
	vector v;
	setVector(&v, value, 0);
	subVector(&b->center, &v);
}

void subYPosition (ball* b, float value) {
	vector v;
	setVector(&v, 0, value);
	subVector(&b->center, &v);
}

void addSpeed (ball* b, vector* v) {
	addVector(&b->speed, v);
}
void subSpeed (ball* b, vector* v) {
	subVector(&b->speed, v);
}

void rotateSpeed (ball* b, float angle) {
	vectorRotation (&b->speed, angle);
}

// Troca de velocidade no eixo x
void tradeSpeed (ball* a, ball* b) {
	vector aux1, aux2;

	setX(&aux1, getX(&b->speed));
	setY(&aux1, getY(&a->speed));

	setX(&aux2, getX(&a->speed));
	setY(&aux2, getY(&b->speed));

	setSpeed(a, &aux1);
	setSpeed(b, &aux2);
}

void resetSpeed (ball* b) {
	vector nullSpeed;
	setVector(&nullSpeed, 0.0, 0.0);
	b->speed = nullSpeed;
}

void reverseXSpeed (ball* b) {
	vector aux;
	setVector(&aux, -1, 1);
	mulVector(&b->speed, &aux);
}

void reverseYSpeed (ball* b) {
	vector aux;
	setVector(&aux, 1, -1);
	mulVector(&b->speed, &aux);
}

int isMoving (ball* b) {
	float vx = getXSpeed(b);
	float vy = getYSpeed(b);

	return ( vx != 0.0 || vy != 0.0 );
}

void putInGame (ball* b) {
	b->game = 1;
}

void kill (ball* b) {
	b->game = 0;
}

void whitePosition (ball* b) {
	vector defaultPosition;
	setVector(&defaultPosition, 135, 210);
	setPosition(b, &defaultPosition);
}

void nextFrame (ball* b) {
	float sqrSpeed;
	sqrSpeed = (getXSpeed(b)*getXSpeed(b)) + (getYSpeed(b)*getYSpeed(b));

	// Muda de frame de acordo com a velocidade
	b->frameCounter += sqrSpeed / ANIMATION_SIZE;

	if (b->frameCounter >= 1.0) {
		b->frameCounter = 0.0;

		b->currentFrame++;

		if (b->currentFrame == ANIMATION_SIZE) {
			b->currentFrame = 0;
		}
	}
}

// Array Functions

// Retorna o numero de bolas restantes em jogo, desconsiderando a bola branca
int ballsLeft (ball* ballArray, int totalBalls) {
	int i, n = 0;
	for (i=1; i < totalBalls; i++) {
		if ( getGameStatus(ballArray+i) ) {
			n++;
		}
	}
	return n;
}

void initializeBallArray (ball* ballArray, int totalBalls) {
	int i, j, k = 0;
	// Inicializa os atributos das bolas
	for(i=0; i < totalBalls; i++) {
		(ballArray+i)->number = i; // identifica o numero da bola
		resetSpeed(ballArray+i); // seta todas as bolas para a velocidade 0;
		(ballArray+i)->speedAngle = 0.0;
		(ballArray+i)->radius = DEFAULT_RADIUS; // seta o raio de todas as bolas como o padrao
		putInGame(ballArray+i); // informa que a bola esta em jogo
		(ballArray+i)->currentFrame = 0; // Animation Frame
		(ballArray+i)->frameCounter = 0.0; // Frame Counter
	}
	
	// Posiciona a bola branca
	whitePosition(ballArray);

	// Inicializa as posicoes das bolas
	vector v;

	for(i=0; i < 5; i++) {
		// Gera o triangulo inicial
		for(j=0; j <= i; j++) {
			setX(&v, i*18+620);
			setY(&v, j*23+211-i*12);
			setPosition(ballArray+1+k, &v);
			k++;
		}
	}
}

// Fucao que libera o espaco alocado pelo malloc e pela funcao load_bitmap
void freeBallArray (ball* ballArray, int totalBalls) {
	int i;
	for(i=0; i < totalBalls; i++) {
		destroy_bitmap(getPicture(ballArray+i));
	}
	free(ballArray);
}

// Client-Server Functions

void importCueBall (ball* b, char* strPos) {
	char x[30];
	char y[30];

	strcpy(x, strPos);
	x[3] = '\0';

	int i;
	for (i=0; i < 3; i++) {
		y[i] = strPos[i+3];
	}
	y[3] = '\0';
	
	setXPosition (b, atof(x));
	setYPosition (b, atof(y));
	putInGame(b);
}

void exportCueBall (ball* b, char* strPos) {
	char x[30];
	char y[30];

	sprintf(x, "%3.0f", getXPosition(b));
	sprintf(y, "%3.0f", getYPosition(b));

	strPos[0] = '\0';
	strcat(strPos, x);
	strcat(strPos, y);
}

