#ifndef _BALL
#define _BALL

#define DEFAULT_RADIUS 10
#define ANIMATION_SIZE 7

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <allegro.h>
#include <string.h>

#include "vector.h"

typedef struct {
	int game;
	int number;

	vector center;
	float radius;

	vector speed;
	float speedAngle;
	
	BITMAP* animation[ANIMATION_SIZE];
	int currentFrame;
	float frameCounter;
} ball;

// Read Properties
int getGameStatus (ball* b);
int getNumber (ball* b);
vector getPosition (ball* b);
float getXPosition (ball* b);
float getYPosition (ball* b);
vector getSpeed (ball* b);
float getXSpeed (ball* b);
float getYSpeed (ball* b);
float getRadius (ball* b);
BITMAP* getPicture (ball* b);
float calculateSpeedAngle (ball* b);
float storeSpeedAngle (ball* b);
float getLastAngle (ball* b);

// Write Properties
void setNumber (ball* b, int num);
void setPosition (ball* b, vector* v);
void setXPosition (ball* b, float x);
void setYPosition (ball* b, float y);
void setSpeed (ball* b, vector* v);
void setRadius (ball* b, float r);
void setPicture (ball* b, BITMAP* p, int frame);

// Position Functions
void addPosition (ball* b, vector* v);
void addXPosition (ball* b, float value);
void addYPosition (ball* b, float value);

void subPosition (ball* b, vector* v);
void subXPosition (ball* b, float value);
void subYPosition (ball* b, float value);

// Speed functions
void addSpeed (ball* b, vector* v);
void subSpeed (ball* b, vector* v);

void rotateSpeed (ball* b, float angle);
void tradeSpeed (ball* a, ball* b);
void resetSpeed (ball* b);
void reverseXSpeed (ball* b);
void reverseYSpeed (ball* b);

int isMoving (ball* b);

// Other
void putInGame (ball* b);
void kill (ball* b);
void whitePosition (ball* b);
void nextFrame (ball* b);

// Array Functions
int ballsLeft (ball* ballArray, int totalBalls);
void initializeBallArray (ball* ballArray, int totalBalls);
void freeBallArray (ball* ballArray, int totalBalls);

// Client-Server Functions
void importCueBall (ball* b, char* strPos);
void exportCueBall (ball* b, char* strPos);

#endif

