/*
  Copyright (C) 2007 Roberto Carraretto & Thiago Fassina Olimpio Carvalho

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include "game.h"

Game* loadGame (int totalPlayers, int totalBalls) {
	Game* game = (Game*) malloc(sizeof(Game));

	// Alocacao de memoria
	game->playerArray = (player*) malloc(sizeof(player)*totalPlayers);
	game->totalPlayers = totalPlayers;
	game->current = 0;
	game->table = (table*) malloc(sizeof(table));
	game->ballArray = (ball*) malloc(sizeof(ball)*totalBalls);
	game->totalBalls = totalBalls;

	// Carrega imagens
	loadTableImage (game->table);
	loadBallArrayImages (game->ballArray, game->totalBalls);
	game->painel = load_bitmap("imagens/painel.bmp",NULL);
	if (!game->painel) {
		printf("Erro ao abrir a imagem!\n");
		exit(1);
	}

	// Carrega sons
	game->sounds = loadSounds();

	return game;
}

void initializeGame (Game* game) {
	int i;
	for (i=0; i < game->totalPlayers; i++) {
		initializePlayer((game->playerArray) + i);
	}
	initializeTable(game->table);
	initializeBallArray(game->ballArray, game->totalBalls);
}

void drawGame (Game* game) {
	refreshScreen(game->table, game->ballArray, game->totalBalls, NULL);
}

void respawnCueBall (Game* game) {
	int i, collision;
	char myKey;

	player* p = (game->playerArray) + game->current;
	table* t = game->table;
	ball* ballArray = game->ballArray;
	int totalBalls = game->totalBalls;

	setAngle(p, DEFAULT_ANGLE);

	whitePosition(ballArray);
	putInGame(ballArray);
	drawStick (p, t, ballArray, totalBalls, 0, 0);

	myKey = readkey();
	while (!key[KEY_SPACE]) {
		if (key[KEY_UP]) {
			if ( ! (collision = checkTableCollision (t, ballArray)) ) {
				subYPosition(ballArray, 1.0);
			
				collision = 1;
				while (collision) {
					for (i=1; i < totalBalls; i++) {
						if ( checkBallCollision (ballArray, (ballArray+i), NULL) ) {
	
							while (checkBallCollision (ballArray, (ballArray+i), NULL)) {
								subYPosition(ballArray, 1.0);
							}
							break; // Vai para o inicio do while
						}
					}
					collision = 0;
				}

				drawStick (p, t, ballArray, totalBalls, 0, 0);
			}
			else {
				fixTableCollision(t, ballArray, collision);
			}
		}

		if (key[KEY_DOWN]) {
			if ( ! (collision = checkTableCollision (t, ballArray)) ) {
				addYPosition(ballArray, 1.0);

				collision = 1;
				while (collision) {
					for (i=1; i < totalBalls; i++) {
						if ( checkBallCollision (ballArray, (ballArray+i), NULL) ) {
	
							while (checkBallCollision (ballArray, (ballArray+i), NULL)) {
								addYPosition(ballArray, 1.0);
							}
							break; // Vai para o inicio do while
						}
					}
					collision = 0;
				}

				drawStick (p, t, ballArray, totalBalls, 0, 0);
			}
			else {
				fixTableCollision(t, ballArray, collision);
			}
		}
	}

	// Arredondamento
	setYPosition(ballArray, (int) getYPosition(ballArray));
	rest(300);
}

// Verifica velocidade das bolas
int turnEnded (ball* ballArray, int totalBalls) {
	int i;
	float vx, vy;

	for(i=0; i < totalBalls; i++) {
		if ( getGameStatus(ballArray+i) ) {
			if (isMoving (ballArray+i)) {
				return 0;
			}
		}
	}
	return 1;
}

void playersTurn(Game* game, int* choice) {

	player* p = (game->playerArray) + game->current;
	table* t = game->table;
	ball* ballArray = game->ballArray;
	int totalBalls = game->totalBalls;

	vector position;

	drawStick (p, t, ballArray, totalBalls, 0, 1);
	show_mouse(screen);

	//NOTE remove next line
	gameConfig config;


	char tecla = readkey();
	while (!key[KEY_SPACE]) {
		if (key[KEY_ESC]) {
			ingame_menu(choice);

			drawStick (p, t, ballArray, totalBalls, 0, 1);

			if (*choice == EXIT) {
				exit(-1);
			}
			if (*choice == MAIN) {
				return;
			}
		}

		if (key[KEY_UP]) {
			if (key[KEY_ALT]) {
				addAngle(p, 0.01);
			}
			else {
				addAngle(p, 0.06);
			}
			drawStick (p, t, ballArray, totalBalls, 0, 1);
		}
		if (key[KEY_DOWN]) {
			if (key[KEY_ALT]) {
				addAngle(p, -0.01);
			}
			else {
				addAngle(p, -0.06);
			}
			drawStick (p, t, ballArray, totalBalls, 0, 1);
		}

		if (key[KEY_M]) {
			setX(&position, mouse_x);
			setY(&position, mouse_y);
			setPosition(ballArray, &position);
			drawStick (p, t, ballArray, totalBalls, 0, 1);
		}

		tecla = readkey();
	}

	float i = 1.57;
	float st = 0;
	rest(10);
	while (!keypressed()) { // seta forca da tacada (taco que balanca)
		drawStick (p, t, ballArray, totalBalls, st, 1);
		st = 21 -(abs(20*(sin(i))) + 1);
		i += 0.05;
		setStr(p, st);
		rest(25);
	}
}

void physicsInterface (Game* game, int* deadBalls, int* fstCollision) {

	table* t = game->table;
	ball* ballArray = game->ballArray;
	int totalBalls = game->totalBalls;
	sounds* snd = &game->sounds;

	int i, j, c, volume, bal;
	int tbCollision, hlCollision;

	int* colArray;
	int colNum;

	for (i = 0; i < totalBalls; i++) {
		if ( getGameStatus(ballArray+i) ) {
			// Movimenta a bola
			movement(t, ballArray+i);

			// Colisao bola-buraco & bola-quinas
			hlCollision = holeAreaCollision (t, ballArray+i);

			switch ( hlCollision ) {
				case 0:
					// Sem colisao
					break;
				case 1:
					// Quina
					// Rotinas de som
					volume = maxVolume(getVolume(getSpeed(ballArray+i), ENERGY_LOSS), 1);
					bal = getBal(getXPosition(ballArray+i));
					play_sample(snd->bolamesa, volume, bal, 1000, FALSE);
					break;
				case 2:
					// Caiu no buraco
					// Rotinas de som
					bal = getBal(getXPosition(ballArray+i));
					play_sample(snd->bolahole, 200, bal, 1000, FALSE);

					//atualiza o vetor de bolas mortas
					if (i) {
						for (c = 0; c < 16; c++) {
							if (!deadBalls[c]) {
								deadBalls[c] = i;
								break;
							}
						}
					}

					resetSpeed(ballArray+i);
					break;
			}

			// Colisao bola-mesa
			tbCollision = tableCollision(t, (ballArray+i));

			if (tbCollision) {
				// Rotinas de som
				volume = maxVolume(getVolume(getSpeed(ballArray+i), ENERGY_LOSS), 1);
				bal = getBal(getXPosition(ballArray+i));
				play_sample(snd->bolamesa, volume, bal, 1000, FALSE);
			}

			// Colisao bola-bola
			colArray = getCollisionOrder (ballArray, i, totalBalls, &colNum);
			for(j=0; j < colNum; j++) {
				ballCollision(t, ballArray+i, ballArray+colArray[j]);

				//indica a primeira colisao da cueBall
				if (!i) {
					if ((*fstCollision) == 0) {
						*fstCollision = colArray[j];
					}
				}

				// Rotinas de som
				volume = maxVolume ( getVolume(getSpeed(ballArray+i), 1.0), getVolume(getSpeed(ballArray+colArray[j]), 1.0) );
				bal = getBal(getXPosition(ballArray+i));
				play_sample(snd->bolabola, volume, bal, 1000, FALSE);
			}
		}
	}
}

// Verifica a situacao da bola branca
int checkCueBallStatus (Game* game) {
	if ( !getGameStatus(game->ballArray) ) { 
		// Revive a bola branca se estiver morta
		respawnCueBall (game);
		rest(300);
		return 1;
	}
	return 0;
}

void executeShot (Game* game, int* fstCollision, int* dead_balls) {

	player* p = (game->playerArray) + game->current;
	table* t = game->table;
	ball* ballArray = game->ballArray;
	int totalBalls = game->totalBalls;
	SAMPLE* shot = game->sounds.bolataco;

	play_sample(shot, 255, 128, 1000, FALSE);
	hitCueBall(p, ballArray);

	// Enquanto ainda tiver algum movimento
	*fstCollision = 0;
	while ( !turnEnded(ballArray, totalBalls) ) {
		// Roda as funcoes de fisica e de atualizacao da tela
		physicsInterface(game, dead_balls, fstCollision);
		drawGame(game);
		rest(4);
	}
	rest(300);
}
