/*
  Copyright (C) 2007 Roberto Carraretto & Thiago Fassina Olimpio Carvalho

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

// Implementacao das funcoes relativas a interface grafica
#include "interface.h"

void initializeInterface() {
	allegro_init(); // funcao padrao do allegro para iniciar a interface
	install_timer();
	install_keyboard();
	install_mouse();

	set_color_depth(16);
	
	if (set_gfx_mode(GFX_AUTODETECT_WINDOWED, 800, 600, 0, 0)) {//verificacao de erro
		printf("Graphic mode error.\n");
		exit(24);
	}

//	if (install_sound(DIGI_ESD, MIDI_NONE, NULL)) {
//		printf("Sound error.\n");
//		exit (25);
//	}
}

void paintItPink (BITMAP* ball) {
	int i;
	int j;
	for (i = 0; i < 20; i++) {
		for (j = 0; j < 20; j++) {
			if (getpixel(ball, i, j) == 55439) {
				putpixel(ball, i, j, makecol(255,0,255));
			}
		}
	}
}

// Carrega a imagem das bolas
void loadBallArrayImages (ball* ballArray, int totalBalls) {
	char* nomeImagem;
	BITMAP* pic;
	
	int i, j;
	for (i = 0; i < totalBalls; i++) {
		//nomeImagem = getImageName(i);

		for (j=0; j < ANIMATION_SIZE; j++) {
			nomeImagem = getImageName (i, j);

			pic = load_bitmap(nomeImagem, NULL);
	
			if (!pic) {
				printf("%s\n", nomeImagem);
				printf("Could not open image.\n");
				exit(0);
			}
	
			paintItPink(pic);
			
	
			setPicture(ballArray+i, pic, j);

			free(nomeImagem);
		}
	}
}

// Obtem o caminho da imagem da bola
char* getImageName(int ballNumber, int frameNumber) {
	char* name = (char*) malloc(sizeof(char)*30);
	char* number = (char*) malloc(sizeof(char)*3);

	if (ballNumber >= 9) {
		number[0] = 'b';
	
		if (ballNumber < 10) {
			number[1] = '0';
			number[2] = '0' + ballNumber;
		}
		else {
			number[1] = '1';
			number[2] = '0' + ballNumber - 10;
		}

		number[3] = '-';
		number[4] = '1' + frameNumber;
		number[5] = '\0';

		strcpy(name, "imagens/");
		strcat(name, number);
		strcat(name, ".bmp");
	
		free(number);
	}
	else {
// 		number[0] = 'b';
// 	
// 		if (ballNumber < 10) {
// 			number[1] = '0';
// 			number[2] = '0' + ballNumber;
// 		}
// 		else {
// 			number[1] = '1';
// 			number[2] = '0' + ballNumber - 10;
// 		}
// 
// 		number[3] = '-';
// 		number[4] = '1' + frameNumber;
// 		number[5] = '\0';

		number[0] = '0' + ballNumber;
		number[1] = '\0';

		strcpy(name, "imagens/bola");
		strcat(name, number);
		strcat(name, ".bmp");
	
		free(number);
	}

	return name;
}

// Carrega a imagem da mesa
void loadTableImage (table* t) {
	BITMAP* pic = load_bitmap("imagens/mesasinuca.bmp",NULL);//carrega a imagem na variável.

	if(!pic){
		printf("Could not open image.\n");
		exit(0);
	}

	setTablePicture(t, pic);
}

// Funcao que desenha os elementos na tela
void refreshScreen (table* t, ball* ballArray, int totalBalls, BITMAP* complement) {

	scare_mouse();

	int i;
	int fixedAngle;
	float angle;
	BITMAP* buffer = create_bitmap(TABLE_PIC_X,TABLE_PIC_Y);
	BITMAP* tPic = getTablePicture(t);
	BITMAP* bPic;
	ball* b;
	
	blit(tPic, buffer, 0, 0, 0, 0, tPic->w, tPic->h);

	for(i=0; i < totalBalls; i++) {
		b = ballArray+i;
		if ( getGameStatus(b) ) {
			bPic = getPicture(b);

			if (isMoving(b)) {
				// Angulo em radianos
				angle = 3.14 - storeSpeedAngle(b);
				// Angulo no formato que sera convertido para fixed ponit (0 a 256)
				// Meia Volta (3.14 rad) -> 128
				fixedAngle = ((angle * 128) / -3.14);
				nextFrame(b);
			}
			else {
				angle = 3.14 - getLastAngle(b);
				fixedAngle = ((angle * 128) / -3.14);
			}

			//draw_sprite(buffer, bPic, getXPosition(b)-(bPic->w)/2, getYPosition(b)-(bPic->h)/2);
			aa_rotate_sprite(buffer, bPic, getXPosition(b), getYPosition(b), itofix(fixedAngle));
			circle(buffer, getXPosition(b), getYPosition(b), DEFAULT_RADIUS, makecol(0,0,0));
		}
	}

	angle += 10;

	if (complement) {
		draw_sprite(buffer, complement, 0, 0);
		destroy_bitmap(complement);
	}

	#ifdef DEBUG
		rect(buffer, R1_X1, R1_Y1, R1_X2, R1_Y2, 0);
		rect(buffer, R2_X1, R2_Y1, R2_X2, R2_Y2, 0);
		rect(buffer, R3_X1, R3_Y1, R3_X2, R3_Y2, 0);
		drawHoles(t, buffer);
		desenhaRetas(buffer, t);
	#endif

	blit(buffer, screen, 0, 0, 0, 0, TABLE_PIC_X, TABLE_PIC_Y);

	destroy_bitmap(buffer);
	unscare_mouse();
	rest(3);

}

void desenhaRetas (BITMAP* buffer, table* t) {
	int i, j;

	for (i=0; i < 6; i++) {
		for (j=0; j < 2; j++) {
			line(buffer, t->rays[i][j].p1.x, t->rays[i][j].p1.y, t->rays[i][j].p2.x, t->rays[i][j].p2.y, makecol(255,255,255));
		}
	}
}

void drawHoles (table* t, BITMAP* buffer) {
	int i;
	vector v;
	for (i = 0; i < 6; i++) {
		v = getHoleCenter(t, i);
		circle(buffer, getX(&v), getY(&v), getHoleRadius(t, i), makecol(255,0,0));
	}
}

void drawStick (player* p, table* t, ball* ballArray, int totalBalls, float forca, int guideLine) {

	float xb, yb;
	BITMAP* buffer = create_bitmap(SCREEN_W, SCREEN_H);
	clear_to_color(buffer, 63519);
	int pol[8];

	xb = getXPosition(ballArray) + 400*cos(getAngle(p));
	yb = getYPosition(ballArray) - 400*sin(getAngle(p));

	pol[0] = getXPosition(ballArray) - 20*cos(getAngle(p) -0.1/((forca+8)/8)) *((forca+8)/8);
	pol[1] = getYPosition(ballArray) + 20*sin(getAngle(p) -0.1/((forca+8)/8)) *((forca+8)/8);

	pol[2] = getXPosition(ballArray) - 20*cos(getAngle(p) +0.1/((forca+8)/8)) *((forca+8)/8);
	pol[3] = getYPosition(ballArray) + 20*sin(getAngle(p) +0.1/((forca+8)/8)) *((forca+8)/8);

	pol[4] = getXPosition(ballArray) - 300*cos(getAngle(p) +0.015/((forca+120)/120)) *((forca+120)/120);
	pol[5] = getYPosition(ballArray) + 300*sin(getAngle(p) +0.015/((forca+120)/120)) *((forca+120)/120);

	pol[6] = getXPosition(ballArray) - 300*cos(getAngle(p) -0.015/((forca+120)/120)) *((forca+120)/120);
	pol[7] = getYPosition(ballArray) + 300*sin(getAngle(p) -0.015/((forca+120)/120)) *((forca+120)/120);

	line (buffer, pol[4], pol[5], pol[6], pol[7], makecol(0,0,0)); // Desenha base do taco
	polygon(buffer, 4, pol, makecol(189, 151, 0)); // Desenha madeira do taco

	pol[0] = getXPosition(ballArray) - 20*cos(getAngle(p) -0.1/((forca+8)/8)) *((forca+8)/8);
	pol[1] = getYPosition(ballArray) + 20*sin(getAngle(p) -0.1/((forca+8)/8)) *((forca+8)/8);

	pol[2] = getXPosition(ballArray) - 20*cos(getAngle(p) +0.1/((forca+8)/8)) *((forca+8)/8);
	pol[3] = getYPosition(ballArray) + 20*sin(getAngle(p) +0.1/((forca+8)/8)) *((forca+8)/8);

	pol[4] = getXPosition(ballArray) - 30*cos(getAngle(p) +0.095/((forca+12)/12)) *((forca+12)/12);
	pol[5] = getYPosition(ballArray) + 30*sin(getAngle(p) +0.095/((forca+12)/12)) *((forca+12)/12);

	pol[6] = getXPosition(ballArray) - 30*cos(getAngle(p) -0.095/((forca+12)/12)) *((forca+12)/12);
	pol[7] = getYPosition(ballArray) + 30*sin(getAngle(p) -0.095/((forca+12)/12)) *((forca+12)/12);

	polygon(buffer, 4, pol, makecol(0,0,0)); // Desenha pontinha do taco

	// Desenha a linha da trajetoria
	if (guideLine) {
		int npontos;
		for (npontos = 0; npontos < 30; npontos++) {
			circle(buffer, getXPosition(ballArray) + (npontos * ((xb-getXPosition(ballArray))/30)), getYPosition(ballArray) + (npontos * (yb-getYPosition(ballArray))/30), 1, makecol(255,255-(12*forca),255-(12*forca)));
		}
	}


// 	line(buffer, getXPosition(ballArray), getYPosition(ballArray), xb, yb, makecol(255,255-(12*forca),255-(12*forca)));

	refreshScreen (t, ballArray, totalBalls, buffer);

}
