/*
  Copyright (C) 2007 Roberto Carraretto & Thiago Fassina Olimpio Carvalho

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include "mod-kk.h"

int refreshScore (Game* game, int* dead_balls, int fstCollision) {

	player* p = game->playerArray + game->current;

	if ( !getGameStatus(game->ballArray) ) {
		printf ("Suicidou-se.\n");
		setScore(p, 0);
		return 0;
	}

	if (!fstCollision) {
		printf ("Nao acertou bolas.\n");
		setScore(p, 0);
		return 0;
	}

	int i = 0;
	while (dead_balls[i]) {
		addScore(p, 1);
		i++;
	}

	return(i);
}

void initializeDeadBalls (int* v) {
	int i;
	for (i = 0; i < 16; i++) {
		v[i] = 0;
	}
}

int kkMatch (Game* game) {
	return (ballsLeft(game->ballArray, game->totalBalls));
}

void kk_single (gameConfig* config) {

	Game* game;

	int deadBalls[16];
	int fstCollision;
	initializeDeadBalls(deadBalls);

	// Aloca memoria, carrega imagens & sons
	game = loadGame(1, 16);
	// Inicializa estruturas
	initializeGame(game);
	// Desenha elementos na tela
	drawGame(game);

	blit(game->painel, screen, 0, 0, 0, 421, 800, 180); //coloca buffer na tela

	// Inicio da partida
	while ( kkMatch(game) ) {

		// Revive a bola branca se estiver morta
		checkCueBallStatus(game);

		// Rodada para o jogador fazer tacada ou abrir menu
		playersTurn(game, &(config->choice));
		if (config->choice == EXIT) {
			exit(1);
		}
		if (config->choice == MAIN) {
			return;
		}
		
		// Executa a jogada
		executeShot(game, &fstCollision, deadBalls);

		// Ajusta o placar
		refreshScore(game, deadBalls, fstCollision);
		initializeDeadBalls(deadBalls);
	}
	// Fim da partida
	config->choice = MAIN;
}


void kk_multi (gameConfig* config) {

	Game* game;
	player* myself;
	player* oponnent;

	int deadBalls[16];
	int fstCollision;
	initializeDeadBalls(deadBalls);

	char dataStr[30];
	int myTurn;
	int socket = config->socket;

	// Aloca memoria, carrega imagens & sons
	game = loadGame(2, 16);
	// Inicializa estruturas
	initializeGame(game);
	// Desenha elementos na tela
	drawGame(game);

	// Decide o turno
	if (config->choice == CREATE) {
		myTurn = 1;
		myself = game->playerArray;
		oponnent = (game->playerArray) + 1;

		//seta os nomes dos players
		strcpy(game->playerArray[0].name, config->playerName);
		envia(game->playerArray[0].name, config->socket);
		recebe(game->playerArray[1].name, config->socket);

	}
	else {
		myTurn = 0;
		myself = (game->playerArray) + 1;
		oponnent = game->playerArray;

		//seta os nomes dos players
		recebe(game->playerArray[0].name, config->socket);
		strcpy(game->playerArray[1].name, config->playerName);
		envia(game->playerArray[1].name, config->socket);

	}

	//NOTE in test
	textout_ex(screen, font, game->playerArray[0].name, 10, 440, makecol(255, 255, 255), -1);
	textout_ex(screen, font, game->playerArray[1].name, 10, 460, makecol(255, 255, 255), -1);

	// Inicio da partida
	while ( kkMatch(game) ) {

		if (myTurn) {
			// Revive a bola branca se estiver morta
			if (checkCueBallStatus(game)) {
				exportCueBall (game->ballArray, dataStr);
				envia (dataStr, socket);
			}
	
			// Rodada para o jogador fazer tacada ou abrir menu
			playersTurn(game, &(config->choice));
			if (config->choice == EXIT) {
				exit(1);
			}
			if (config->choice == MAIN) {
				return;
			}
			// Envia dados da tacada para o adversario
			exportShot(myself, dataStr);
			envia(dataStr, socket);
		}
		else {
			// Recebe dados do adversario
			if (!getGameStatus(game->ballArray)) {
				// Recebe posicao da bola branca
				recebe(dataStr, socket);
				importCueBall(game->ballArray, dataStr);
			}
			// Recebe tacada
			recebe(dataStr, socket);
			importShot(oponnent, dataStr);
		}
		
		// Executa a jogada
		executeShot(game, &fstCollision, deadBalls);

		// Ajusta o placar
		if (!refreshScore(game, deadBalls, fstCollision)) {
			// Se nao marcou ponto, passa a vez
			game->current = !(game->current);
			myTurn = !myTurn;
		}

		initializeDeadBalls(deadBalls);
	}
	// Fim da partida
	config->choice = MAIN;
}



void mod_kk (gameConfig* config) {

	if (config->choice == SINGLE) {
		printf ("Kill Kill (single player) starting,\n");
		kk_single(config);
		return;
	}
	else {

		if (config->choice == CREATE) {
			envia("00", config->socket);
		}

		printf ("Kill Kill (multiplayer) starting,\n");
		kk_multi(config);
		return;
	}
}
