/*
  Copyright (C) 2007 Roberto Carraretto

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include "physics.h"

// TODO(?): Perder energia na colisao bola-bola?

// Atrito
void attrition (table* t, ball *b) {
	if (isMoving(b)) {
		float attrict = getAttrition(t);
		vector speed = getSpeed(b);
	
		float angle = calculateSpeedAngle(b);
		float oldX;
	
		vectorRotation(&speed, angle);
		oldX = getX(&speed);
	
		if (oldX == 0) {
			return;
			
		}
		else if (oldX > 0) {
			setX(&speed, oldX - attrict);
		}
		else {
			setX(&speed, oldX + attrict);
		}
	
		if (oldX * getX(&speed) < 0) {
			// Inverteu a velocidade
			resetSpeed(b);
		}
		else {
			vectorRotation(&speed, -angle);
			setSpeed(b, &speed);
		}
	}
}

// Perda de energia
void energyLoss (ball* b) {

	vector ballSpeed = getSpeed(b);
	vector loss;
	setVector(&loss, ENERGY_LOSS, ENERGY_LOSS);
	
	mulVector(&ballSpeed, &loss);
	setSpeed(b, &ballSpeed);
}

// Deslocamento
void movement (table* t, ball* b) {
	vector ballSpeed;

	attrition(t, b);
	ballSpeed = getSpeed(b);
	addPosition (b, &ballSpeed);
}

// Deslocamento Inverso
void inverseMovement (table* t, ball* b) {
	vector ballSpeed = getSpeed(b);
	subPosition (b, &ballSpeed);
}

// Insere ordenado em um vetor de distancias (Retorna a posicao em que o elemento foi inserido)
int insertDistance (float* distArray, int size, float dist) {
	int i, j;

	for (i=0; i < size; i++) {
		if (dist <= distArray[i]) {
			for (j=size; j > i; j--) {
				distArray[i] = distArray[i-1];
			}
			distArray[i] = dist;
			break;
		}
	}

	return i;
}

// Insere em uma posicao especifica de um vetor de index
void insertIndex (int* indexArray, int size, int value, int position) {
	int i;

	for (i=size; i > position; i--) {
		indexArray[i] = indexArray[i-1];
	}
	indexArray[i] = value;
}

// Retorna um vetor com a ordem da colisao
int* getCollisionOrder (ball* ballArray, int ballIndex, int totalBalls, int* totalCollisions) {
	int i, position;
	int* indexArray = (int*) malloc(sizeof(int)*(totalBalls-ballIndex));
	float* distArray = (float*) malloc(sizeof(float)*(totalBalls-ballIndex));
	float dist;

	*totalCollisions = 0;

	for (i=ballIndex+1; i < totalBalls; i++) {
		if ( getGameStatus(ballArray+i) ) {
			if ( checkBallCollision (ballArray+ballIndex, ballArray+i, &dist) ) {
				position = insertDistance (distArray, *totalCollisions, dist);
				insertIndex (indexArray, *totalCollisions, i, position);
				*totalCollisions = *totalCollisions + 1;
			}
		}
	}

	return indexArray;
}

// Auxiliar para intersecao de esfera e segmento de reta
int checkIntersection (ray* r, point* p) {
	point p0, p1, p2;
	float x0, x1, x2;

	p0 = *p;
	p1 = getP1(r);
	p2 = getP2(r);

	vectorRotation(&p0, getRayAngle(r));
	vectorRotation(&p1, getRayAngle(r));
	vectorRotation(&p2, getRayAngle(r));

	x0 = getX(&p0);
	x1 = getX(&p1);
	x2 = getX(&p2);

	return ( (x1 <= x0 && x0 <= x2) || (x2 <= x0 && x0 <= x1) );
}

// Colisao bola-bola
int checkBallCollision (ball* a, ball* b, float* pDist) {
	float sqrDist, sqrRadiusSum, radiusSum;
	vector aPos, bPos;

	aPos = getPosition(a);
	bPos = getPosition(b);

	sqrDist = sqrDistance(&aPos, &bPos);
	radiusSum = getRadius(a) + getRadius(b);
	sqrRadiusSum = radiusSum * radiusSum;

	if (pDist)
		*pDist = sqrDist;

	if (sqrDist <= sqrRadiusSum) {
		return (1);
	} else{
		return (0);
	}
}

// Colisao bola-mesa
int checkTableCollision (table* t, ball* b) {
	int collision = 0;

	if ( !holeArea(getPosition(b)) ) {
		// Canto direito
		if ( getXPosition(b) + getRadius(b) >= getXPositionTb(t) + getXDimension(t) ) {
			collision = 1;
		}
		// Canto esquerdo
		else if ( getXPosition(b) - getRadius(b) <= getXPositionTb(t) ) {
			collision = 2;
		}
	
		// Canto inferior
		else if ( getYPosition(b) + getRadius(b) >= getYPositionTb(t) + getYDimension(t) ) {
			collision = 3;
		}
		
		// Canto superior
		else if ( getYPosition(b) - getRadius(b) <= getYPositionTb(t) ) {
			collision = 4;
		}
	}

	return(collision);
}

// Colisao bola-buraco
int checkHoleCollision (table* t, ball* b, int hole) {
	point p1, p2;
	float sqrDist, sqrRadius;

	p1 = getHoleCenter(t, hole);
	p2 = getPosition(b);

	sqrDist = sqrDistance(&p1, &p2);
	sqrRadius = HOLE_RADIUS * HOLE_RADIUS;

	if (sqrDist <= sqrRadius) {
		return 1;
	}

	return 0;
}

// Colisao bola-bordas
int checkBorderCollision (table* t, ball* b, int hole, int* index) {
	point p0, p2;
	ray r1, r2;
	float sqrDist, radius;

	p0 = getPosition(b);
	r1 = getRay(t, hole, 0);
	r2 = getRay(t, hole, 1);

	radius = getRadius(b);

	// Reta 1
	sqrDist = sqrPointRayDistance(&r1,&p0);
	if ( sqrDist <= (radius * radius) ) {
		if ( checkIntersection (&r1, &p0) ) {
			*index = 0;
			return 1;
		}
	}
	
	// Reta 2
	sqrDist = sqrPointRayDistance(&r2,&p0);
	if ( sqrDist <= (radius * radius) ) {
		if ( checkIntersection (&r2, &p0) ) {
			*index = 1;
			return 1;
		}
	}

	// Quina 1
	p2 = getP2(&r1);
	sqrDist = sqrDistance(&p0, &p2);
	if ( sqrDist <= (radius * radius) ) {
		*index = 0;
		return 2;
	}

	// Quina 2
	p2 = getP2(&r2);
	sqrDist = sqrDistance(&p0, &p2);
	if ( sqrDist <= (radius * radius) ) {
		*index = 1;
		return 2;
	}

	return 0;
}

// Colisao bola-bola
void fixBallCollision (ball* a, ball* b) {
	// NOTE: As bolas devem ter sido rotacionadas

	// Enquanto houver colisao entre o mesmo par de bolas
	while (checkBallCollision(a, b, NULL)) {
		if ( getXPosition(a) > getXPosition(b) ) {
			addXPosition(a, 1.0);
			subXPosition(b, 1.0);
		}
		else {
			addXPosition(b, 1.0);
			subXPosition(a, 1.0);
		}
	}
}

// Colisao bola-mesa
void fixTableCollision (table* t, ball* b, int collision) {

	switch ( collision ) {
		case 1:
			// Canto direito
			setXPosition(b, getXPositionTb(t) + getXDimension(t) - getRadius(b) - FACTOR);
			break;
		
		case 2:
			// Canto esquerdo
			setXPosition(b, getXPositionTb(t) + getRadius(b) + FACTOR);
			break;
	
		case 3:
			// Canto inferior
			setYPosition(b, getYPositionTb(t) + getYDimension(t) - getRadius(b) - FACTOR);
			break;
	
		case 4:
			// Canto superior
			setYPosition(b, getYPositionTb(t) + getRadius(b) + FACTOR);
			break;

		default:
			break;
	}
}

// Colisao bola-borda
void fixBorderCollision (ball* b, point p0, float angle) {
	vector bp;
	bp = getPosition(b);

	vectorRotation (&bp, angle);
	vectorRotation (&p0, angle);

	if ( getY(&bp) > getY(&p0) ) {
		setY(&bp, getY(&p0) + getRadius(b) + FACTOR);
	}
	else {
		setY(&bp, getY(&p0) - getRadius(b) - FACTOR);
	}

	vectorRotation (&bp, -angle);

	setPosition(b, &bp);
}

// Colisao bola-quina
void fixBorderCollision2 (ball* b, point p0, float angle) {
	vector bp;
	bp = getPosition(b);

	vectorRotation (&bp, angle);
	vectorRotation (&p0, angle);

	if ( getX(&bp) > getX(&p0) ) {
		setX(&bp, getX(&p0) + getRadius(b) + FACTOR);
	}
	else {
		setX(&bp, getX(&p0) - getRadius(b) - FACTOR);
	}

	vectorRotation (&bp, -angle);

	setPosition(b, &bp);
}

// Colisao bola-bola
void ballCollision (table* t, ball* a, ball* b) {
	float angle;
	vector aCenter, bCenter;

	// Angulo de colisao em relacao ao eixo x
	aCenter = getPosition(a);
	bCenter = getPosition(b);
	angle = calculateAngle(&aCenter, &bCenter);

	// Rotacionando vetores
	rotateSpeed (a, angle);
	rotateSpeed (b, angle);

	// Troca de valores no eixo x
	tradeSpeed (a, b);

	// Tira do estado de colisao
	fixBallCollision (a, b);

	// Desfaz rotacao
	rotateSpeed (a, -angle);
	rotateSpeed (b, -angle);
}

// Colisao bola-mesa
int tableCollision (table* t, ball* b) {

	int inHoleArea;
	int collision;

	int hole, index;
	float angle;
	ray r;

	collision = checkTableCollision (t, b);

	switch (collision) {
		case 1:
			// Canto direito
			reverseXSpeed(b);
			break;
		case 2:
			// Canto esquerdo
			reverseXSpeed(b);
			break;
		case 3:
			// Canto inferior
			reverseYSpeed(b);
			break;
		case 4:
			// Canto superior
			reverseYSpeed(b);
			break;
		default:
			break;
	}

	if (collision) {
		energyLoss(b);
		// Tira do estado de colisao
		fixTableCollision (t, b, collision);
	}

	return(collision);
}

// Colisao bola-bordas & bola-buraco
int holeAreaCollision (table* t, ball* b) {
	int collision, hole, index;
	float angle;
	ray r;
	point p0, p2;

	if ( !holeArea(getPosition(b)) ) {
		// Fora da regiao de risco
		return 0;
	}

	// Buraco da possivel colisao
	hole = nearestHole(getPosition(b));
	collision = checkHoleCollision (t, b, hole);

	if (collision) {
		// Colisao com buraco
		kill(b);
		return 2;
	}

	collision = checkBorderCollision (t, b, hole, &index);

	if ( collision ) {
		r = getRay (t, hole, index);

		if ( collision == 1 ) {
			// Colisao com alguma borda
			angle = getRayAngle(&r);
			p2 = getP2(&r);
		}
		else {
			// Colisao com alguma quina
			p0 = getPosition(b);
			p2 = getP2(&r);
			angle = calculateAngle(&p0, &p2);
		}

		rotateSpeed (b, angle);
		if ( collision == 1 ) {
			// Colisao com alguma borda
			fixBorderCollision(b, p2, angle);
			reverseYSpeed(b);
		}
		else {
			// Colisao com alguma quina
			fixBorderCollision2(b, p2, angle);
			reverseXSpeed(b);
		}
		rotateSpeed (b, -angle);

		energyLoss(b);

		return 1;
	}

	return 0;
}

// Faz a bola branca adquirir velocidade
void hitCueBall(player* p, ball* cueBall) {
	vector v;

	setX(&v, getStr(p)*cos(getAngle(p)));
	setY(&v, getStr(p)*sin(getAngle(p)*-1));
	setSpeed(cueBall, &v);
}
