/*
  Copyright (C) 2007 Roberto Carraretto & Thiago Fassina Olimpio Carvalho

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include "table.h"

// Read Properties
float getAttrition (table* t) {
	return (t->attrition);
}

vector getPositionTb (table* t) {
	return (t->position);
}
float getXPositionTb (table* t) {
	return (getX(&t->position));
}

float getYPositionTb (table* t) {
	return (getY(&t->position));
}

vector getDimension (table* t) {
	return (t->dimension);
}

float getXDimension (table* t) {
	return (getX(&t->dimension));
}

float getYDimension (table* t) {
	return (getY(&t->dimension));
}

vector getHoleCenter (table* t, int holeNum) {
	return (t->holes[holeNum].center);
}

float getHoleRadius (table* t, int holeNum) {
	return (t->holes[holeNum].radius);
}

BITMAP* getTablePicture (table* t) {
	return (t->picture);
}

ray getRay (table* t, int hole, int index) {
	return (t->rays[hole][index]);
}

// Write Propertis
void  setTablePicture (table* t, BITMAP* p) {
	t->picture = p;
}

// Other
void initializeTable (table* t) {
	setX(&t->position, X_POSITION);
	setY(&t->position, Y_POSITION);

	setX(&t->dimension, X_DIMENSION);
	setY(&t->dimension, Y_DIMENSION);

	t->attrition = DEFAULT_ATTRITION;

	int i;
	for (i=0; i < 6; i++) {
		t->holes[i].radius = HOLE_RADIUS;
	}

	setX(&t->holes[0].center, 24);
	setY(&t->holes[0].center, 26);

	setX(&t->holes[1].center, 399);
	setY(&t->holes[1].center, 21);

	setX(&t->holes[2].center, 774);
	setY(&t->holes[2].center, 26);

	setX(&t->holes[3].center, 24);
	setY(&t->holes[3].center, 394);

	setX(&t->holes[4].center, 399);
	setY(&t->holes[4].center, 399);

	setX(&t->holes[5].center, 774);
	setY(&t->holes[5].center, 394);

	// Padrao das retas
	// p1 - ponto mais perto do buraco
	// p2 - ponto mais longe do buraco

#define X0  20
#define Y0  20
#define X1  778
#define Y1  400
#define H_X (X_POSITION + X_DIMENSION)
#define H_Y (Y_POSITION + Y_DIMENSION)

	vector center;

	center = getHoleCenter(t, 0);
	initializeRay(&t->rays[0][0], getX(&center)+HOLE_RADIUS, Y0, R2_X1, Y_POSITION);
	initializeRay(&t->rays[0][1], X0, (getY(&center)+HOLE_RADIUS), X_POSITION, R1_Y1);

	center = getHoleCenter(t, 1);
	initializeRay(&t->rays[1][0], (getX(&center)-HOLE_RADIUS+1), Y0, (getX(&center)-getHoleRadius(t, 1)-7), Y_POSITION);
	initializeRay(&t->rays[1][1], (getX(&center)+HOLE_RADIUS), Y0, (getX(&center)+getHoleRadius(t, 1)+7), Y_POSITION);

	center = getHoleCenter(t, 2);
	initializeRay(&t->rays[2][0], (getX(&center)-HOLE_RADIUS), Y0, R3_X2, Y_POSITION);
	initializeRay(&t->rays[2][1], X1, (getY(&center)+HOLE_RADIUS), H_X, R1_Y1);

	center = getHoleCenter(t, 3);
	initializeRay(&t->rays[3][0], X0, (getY(&center)-HOLE_RADIUS), X_POSITION, R1_Y2);
	initializeRay(&t->rays[3][1], (getX(&center)+HOLE_RADIUS), Y1, R2_X1, H_Y);

	center = getHoleCenter(t, 4);
	initializeRay(&t->rays[4][0], (getX(&center)-HOLE_RADIUS+1), Y1, (getX(&center)-getHoleRadius(t, 4)-7), H_Y);
	initializeRay(&t->rays[4][1], (getX(&center)+HOLE_RADIUS), Y1, (getX(&center)+getHoleRadius(t, 4)+7), H_Y);

	center = getHoleCenter(t, 5);
	initializeRay(&t->rays[5][0], X1, (getY(&center)-HOLE_RADIUS), H_X, R1_Y2);
	initializeRay(&t->rays[5][1], (getX(&center)-HOLE_RADIUS), Y1, R3_X2, H_Y);
}

void freeTable (table* t) {
	destroy_bitmap(getTablePicture(t));
	free(t);
}

// Verifica se esta perto de algum buraco
int holeArea (point p) {
	float x = getX(&p);
	float y = getY(&p);

	if ((x > R1_X1 && x < R1_X2) && (y > R1_Y1 && y < R1_Y2)) {
		// Regiao 1
		return(0);
	}
	if ((x > R2_X1 && x < R2_X2) && (y > R2_Y1 && y < R2_Y2)) {
		// Regiao 2
		return(0);
	}
	if ((x > R3_X1 && x < R3_X2) && (y > R3_Y1 && y < R3_Y2)) {
		// Regiao 3
		return(0);
	}
	// Area de buraco
	return(1);
}

// Buraco mais proximo de um ponto
int nearestHole (point p) {
	int x, y;
	x = getX(&p);
	y = getY(&p);

	if (y < 210) {
		if (x < 100) {
			return(0);
		}
		if (x > 700) {
			return(2);
		}
		return(1);
	}
	if (x < 100) {
		return(3);
	}
	if (x > 700) {
		return(5);
	}
	return(4);
}
